//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This example demonstrates how to use openGL automatic 
///			texture cordinates to project a texture onto an object.
///			uses include underwater effects, projected lights etc.
//------------------------------------------------------------

#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include "texture.h"

/// this will hold the openGL texture object for the loaded texture
unsigned int g_Texture=0;

/// the position of the object
float position[2] = {0,0};


//------------------------------------------------------------	Draw()
//
void OnDraw() {

	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear previous transformations
	glLoadIdentity();

	// set the camera position
	gluLookAt(0,0,10,0,0,0,0,1,0);

	// bind the environment map texture
	glBindTexture(GL_TEXTURE_2D,g_Texture);

	// move to the correct position
	glTranslatef(position[0],position[1],0);
	
	// draw the teapot
	glutSolidTeapot(1);

	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {

	// enable depthe testing
	glEnable(GL_DEPTH_TEST);

	// enable texturing
	glEnable(GL_TEXTURE_2D);

	// tell openGL to generate the texture coords for a sphere map
	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);

	// enable automatic texture coordinate generation
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);

	// enable a simple light
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// load texture as compressed
	g_Texture = LoadTexture("texture.bmp");

	// print some info about it
	std::cout << "Texture Size=" << (GetTextureSize(g_Texture)/1024.0f) << "Kb"<< std::endl;
}

//------------------------------------------------------------	Reshape()
void OnReshape(int w, int h)
{
	// prevent division by zero when minimised
	if (h==0) 
		h=1;
	glViewport(0,0,w,h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,(float)w/h,0.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


//------------------------------------------------------------	OnExit()
//
void OnExit() {
	DeleteAllTextures();
}

void OnKey(int key,int x,int y) {

	// move object with the cursor keys
	switch(key) {
	case GLUT_KEY_UP:
		position[1] += 0.1f;
		break;
	case GLUT_KEY_DOWN:
		position[1] -= 0.1f;
		break;
	case GLUT_KEY_RIGHT:
		position[0] += 0.1f;
		break;
	case GLUT_KEY_LEFT:
		position[0] -= 0.1f;
		break;
	default:
		break;
	}
	glutPostRedisplay();
}


//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);
	glutInitWindowSize(640,480);
	glutCreateWindow("Projected Texture Example");

	glutDisplayFunc(OnDraw);
	glutReshapeFunc(OnReshape);
	glutSpecialFunc(OnKey);
	
	OnInit();

	atexit(OnExit);

	glutMainLoop();
	return 0;
}
